<?php

namespace App\Controllers;

use Hleb\Constructor\Handlers\Request;
use App\Models\SearchModel;
use UserData, Meta;

class SearchController extends Controller
{
    protected $limit = 10;

    public function index()
    {
        return view(
            '/default/content/search/home',
            [
                'meta'  => Meta::get(__('search.title'), __('search.desc', ['name' => config('meta.name')])),
            ]
        );
    }

    public function openSearch()
    {
        return includeCachedTemplate('/default/content/search/open-search');
    }

    public function go()
    {
        $q      = Request::getGet('q');
        $type   = Request::getGet('cat');

        if (!in_array($type, ['post', 'website', 'answer'])) {
            $type = 'post';
        }

        $sw = microtime(true);

        if ($q) {

            $lang = config('general.lang');
            if (!in_array($lang, ['ru', 'en'])) {
                $lang = 'en';
            }

            $results = SearchModel::getSearch($this->pageNumber, $this->limit, $q, $type);
            $count_results =  SearchModel::getSearchCount($q, $type);

            $user_id = UserData::getUserId();
            SearchModel::setSearchLogs(
                [
                    'request'       => $q,
                    'action_type'   => $type,
                    'add_ip'        => Request::getRemoteAddress(),
                    'user_id'       => $user_id > 0 ? $user_id : 1,
                    'count_results' => $count_results,
                ]
            );
        }

        $count = $count_results ?? 0;

        $facet = $type == 'post' ? 'topic' : 'category';
        return $this->render(
            '/search/search',
            [
                'meta'  => Meta::get(__('search.title')),
                'data'  => [
                    'results'       => $results ?? false,
                    'type'          => $type,
                    'sheet'         => 'admin',
                    'q'             => $q ?? null,
                    'tags'          => SearchModel::getSearchTags($q ?? null, $facet, 4),
                    'sw'            => round((microtime(true) - $sw ?? 0) * 1000, 4),
                    'count'         => $count,
                    'pagesCount'    => ceil($count / $this->limit),
                    'pNum'          => $this->pageNumber,
                ]
            ],
            'search',
        );
    }

    public function api()
    {
        $query  = Request::getPost('query');
        $search = preg_replace('/[^a-zA-ZА-Яа-я0-9 ]/ui', '', $query);

        $belong  = Request::getPost('type');
        $type    = $belong == 'topic' ? 'topic' : 'category';
        $content = $type == 'topic' ? 'post' : 'website';

        $topics = SearchModel::getSearchTags($search, $type, 3);
        $posts  = SearchModel::getSearch(1, 5, $search, $content);
        $result = array_merge($topics, $posts);

        return json_encode($result, JSON_PRETTY_PRINT);
    }

    // Related posts, content author change, facets 
    // Связанные посты, изменение автора контента, фасеты
    public function select()
    {
        $type       = Request::get('type');
        $search     = Request::getPost('q');

        return SearchModel::getSelect($search, $type);
    }
}
